// Copyright 2024 The Bazel Authors. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package com.google.devtools.build.lib.skyframe.serialization;

/** Provides key bytes for {@link FingerprintValueStore}. */
public sealed interface KeyBytesProvider permits PackedFingerprint, StringKey {
  /** Produces the {@code byte[]} representation of the key. */
  byte[] toBytes();

  /** Concatenates {@code bytes} to the {@code byte[]} representation of this key. */
  byte[] concat(byte[] bytes);
}
